==> migrate_wizard-1.0.19-alpha1.tar.gz: migrate_wizard/src/Plugin/migrate/source/d7/MWD7Node.php <==
<?php

namespace Drupal\migrate_wizard\Plugin\migrate\source\d7;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\migrate\Row;
use Drupal\migrate_drupal\Plugin\migrate\source\d7\FieldableEntity;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\migrate_wizard\FieldTypeMWManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\node\Plugin\migrate\source\d7\Node;

/**
 * Drupal 7 node source from database.
 *
 * Available configuration keys:
 * - node_type: The node_types to get from the source - can be a string or
 *   an array. If not declared then nodes of all types will be retrieved.
 *
 * Examples:
 *
 * @code
 * source:
 *   plugin: d7_node
 *   node_type: page
 * @endcode
 *
 * In this example nodes of type page are retrieved from the source database.
 *
 * @code
 * source:
 *   plugin: d7_node
 *   node_type: [page, test]
 * @endcode
 *
 * In this example nodes of type page and test are retrieved from the source
 * database.
 *
 * For additional configuration keys, refer to the parent classes.
 *
 * @see \Drupal\migrate\Plugin\migrate\source\SqlBase
 * @see \Drupal\migrate\Plugin\migrate\source\SourcePluginBase
 *
 * @MigrateSource(
 *   id = "mw_d7_node",
 *   source_module = "node"
 * )
 */
class MWD7Node extends Node {

  /**
   * The config of content type to import.
   *
   * @var string
   */
  protected $config;

  /**
   * The config of content type to import.
   *
   * @var string
   */
  protected $configFactory;

  /**
   * Array of bundles.
   *
   * @var array
   */
  public $bundle;

  /**
   * Array of nid or vid destiny type.
   *
   * @var string
   */
  protected $destinyType;

  /**
   * Array of machine name of origin to import.
   *
   * @var string
   */
  protected $originType;

  /**
   * The left join table.
   *
   * @var string
   */
  protected $leftJoinTable;

  /**
   * The current config of content to import.
   *
   * @var array
   */
  protected $currentConfig;


  /**
   * {@inheritdoc}
   */
  protected $fieldTypeMWManager;

  /**
   * {@inheritdoc}
   */
  protected $leftTable;

  /**
   * {@inheritdoc}
   */
  protected $leftTableId;

  /**
   * {@inheritdoc}
   */
  protected $entityType;

  /**
   * {@inheritdoc}
   */
  protected $translation;
  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration,
                              $plugin_id,
                              $plugin_definition,
                              MigrationInterface $migration,
                              StateInterface $state,
                              EntityTypeManagerInterface $entity_type_manager,
                              ModuleHandlerInterface $module_handler,
                              ConfigFactoryInterface $config_factory,
                              FieldTypeMWManagerInterface $field_type_mw_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration, $state, $entity_type_manager, $module_handler);

    $this->state = $state;
    $current_source_config = $migration->getSourceConfiguration();
    $this->config = $config_factory->getEditable('migrate_wizard.settings');
    $this->bundle = $current_source_config['bundle'];
    if(isset($current_source_config["constants"]["translations"]) && $current_source_config["constants"]["translations"]){
      $this->bundle = str_replace('_'.$current_source_config["constants"]["langcode"], '', $current_source_config['bundle']);
    }
    $this->currentConfig = $this->config->get($this->bundle);

    $this->translation = ['und', 'es'];
    if(isset($this->currentConfig[array_key_first($this->currentConfig)]["languages"])){
      $this->translation = ['und', $this->currentConfig[array_key_first($this->currentConfig)]["languages"]["default"]];
    }

    if(isset($configuration["constants"]["translations"]) && $configuration["constants"]["translations"]){
      $this->translation = [$configuration["constants"]["langcode"]];
    }
    $this->destinyType = $this->currentConfig["destiny_type"];
    $this->originType = $this->currentConfig["origin_type"];
    $this->configFactory = $config_factory;
    $this->fieldTypeMWManager = $field_type_mw_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration = NULL) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $migration,
      $container->get('state'),
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('config.factory'),
      $container->get('plugin.manager.field_type_mw')
    );
  }

  /**
   * The join options between the node and the node_revisions table.
   */
  const JOIN = '[n].[vid] = [nr].[vid]';

  /**
   * {@inheritdoc}
   */
  public function query() {

    $this->leftTable = 'n';
    $this->leftTableId = 'nid';
    $this->entityType = 'node';
    $this->leftJoinTable = $this->getLeftJoinTable();


    $query = parent::query();

    if($this->currentConfig["origin_type"] == 'node'){
      $query->condition('n.language', $this->translation, 'in');
    }

    if (isset($this->currentConfig["import_path_alias"]) && $this->currentConfig["import_path_alias"] == 1 && !array_key_exists('ual', $query->getTables())) {
      $this->queryEntityUrlAlias($query);
    }
    if (isset($this->currentConfig["source_type_date_field"]) && $this->currentConfig["source_type_date_field"] != NULL) {
      $this->addFromCreate($query, strtotime($this->currentConfig["source_type_date_field"]));
    }

    if (isset($this->currentConfig["import_only_ids"]) && $this->currentConfig["import_only_ids"] != NULL) {
      $this->importOnlyIds($query);
    }

    if (isset($this->currentConfig["limit_import"]) && $this->currentConfig["limit_import"] != NULL) {
      $this->limitImport($query);
    }

    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareRow(Row $row) {

    $prepare_row_result = parent::prepareRow($row);

    if($row->hasSourceProperty('translations') && $row->getSourceProperty('translations')){
       $row->setSourceProperty('default_langcode', 0);
    }
    // Get field API field values.
    foreach (array_keys($this->getFields($this->currentConfig["origin_type"], $this->bundle)) as $field) {
      if(isset(reset($this->currentConfig)[$field]) && (reset($this->currentConfig)[$field]['type'] == 'field_collection') || (reset($this->currentConfig)[$field]['type'] == 'paragraphs')){
        $new_field_source = [];
        $field_source = $row->getSourceProperty($field);
        foreach($field_source as $field_source_key => $field_source_value){
          $new_field_source[$field_source_key] = ['item_id' => $field_source_value['value'], 'revision_id' => $field_source_value['revision_id']];
        }
        $row->setSourceProperty($field, $new_field_source);
      }
    }

    foreach ($this->currentConfig[array_key_first($this->currentConfig)] as $field_name => $data_field) {
      if(isset($data_field['type']) && $data_field["destiny"] != 'none'){
        switch ($data_field['type']) {
          case 'image':
          case 'file':
            $new_value = [];
            foreach($row->getSourceProperty($field_name) as $key => $value){
              $query_uri = $this->select('file_managed', 'fm');
              $query_uri->fields('fm', ['uri']);
              $query_uri->condition('fm.fid', $value['fid']);
              $uri = $query_uri->execute()->fetchField();
              $new_value[$key] = $value;
              $new_value[$key]['uri'] = $uri;
              $new_value[$key]['type'] = $data_field['type'];
            }
            $row->setSourceProperty($field_name, $new_value);
            break;
          case 'datetime':
            $date = $row->getSourceProperty($field_name . '_value');
            if (isset($data_field['format']) && $data_field['format'] != '' && $date != '') {
              $date = DrupalDateTime::createFromFormat('Y-m-d H:i:s', $date);
              $date->format($data_field['format']);
              $row->setSourceProperty($field_name . '_value', $date->format($data_field['format']));

            }
            break;
          case 'list_text':
            if ($data_field['destiny_type'] == 'entity_reference') {
              $this->listTextToTaxonomy($row, $field_name, $data_field);
            }
            break;
        }
      }
    }

    return $prepare_row_result;
  }

  /**
   * {@inheritdoc}
   */
  protected function queryEntityUrlAlias(&$query): void
  {

    $translations = implode("','", $this->translation);
    $id__left = $this->leftJoinTable;
    if($this->leftJoinTable == 'tv.vid'){
      $id__left = 'ttd.tid';
    }

    $source = $this->entityType;
    if($this->entityType == 'taxonomy_term'){
      $source = 'taxonomy/term';
    }

    $query->leftJoin('url_alias', 'ual', "CONCAT('{$source}/', $id__left) = ual.source AND ual.pid = (SELECT MAX(ual2.pid) FROM url_alias as ual2 WHERE ual2.source=CONCAT('{$source}/', $id__left)) AND ual.language in('$translations')");
    $query->fields('ual', ['alias']);
    $query->groupBy('ual.alias');
  }

  /**
   * {@inheritdoc}
   */
    protected function addFromCreate(&$query, $timestamp): void
  {
    $query->condition('created', $timestamp, '>=');
  }

  /**
   * {@inheritdoc}
   */
    protected function importOnlyIds(&$query): void
  {
    switch($this->currentConfig["origin_type"]){
      case 'node':
        $query->condition('n.nid', explode(',', $this->currentConfig["import_only_ids"]), 'IN');
        break;
      case 'taxonomy_term':
        $query->condition('ttd.tid', explode(',', $this->currentConfig["import_only_ids"]), 'IN');
        break;
      case 'user':
        $query->condition('u.uid', explode(',', $this->currentConfig["import_only_ids"]), 'IN');
        break;
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function limitImport(&$query): void
  {
    $query->range(0, $this->currentConfig["limit_import"]);

    $query->orderBy($this->leftJoinTable, 'ASC');
  }

  /**
   * {@inheritdoc}
   */
  protected function getLeftJoinTable(): string
  {
    return "$this->leftTable.$this->leftTableId";
  }

  /**
   * {@inheritdoc}
   */
  private function listTextToTaxonomy(&$row, $field_name, $data_field) {
    // Load taxonomy term.
    $term = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadByProperties(['name' => $row->getSourceProperty($field_name . '_value')]);
    $term = reset($term);
    if ($term == FALSE) {
      $term = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->create([
        'name' => $row->getSourceProperty($field_name . '_value'),
        'vid' => $data_field['destiny'],
      ]);
      $term->save();
    }
    $row->setSourceProperty($field_name . '_value', $term->id());
  }
}

==> migrate_wizard-1.0.21-alpha1.tar.gz: migrate_wizard/src/Plugin/migrate/source/d7/MWD7Node.php <==
<?php

namespace Drupal\migrate_wizard\Plugin\migrate\source\d7;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\migrate\Row;
use Drupal\migrate_drupal\Plugin\migrate\source\d7\FieldableEntity;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\migrate_wizard\FieldTypeMWManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\node\Plugin\migrate\source\d7\Node;

/**
 * Drupal 7 node source from database.
 *
 * Available configuration keys:
 * - node_type: The node_types to get from the source - can be a string or
 *   an array. If not declared then nodes of all types will be retrieved.
 *
 * Examples:
 *
 * @code
 * source:
 *   plugin: d7_node
 *   node_type: page
 * @endcode
 *
 * In this example nodes of type page are retrieved from the source database.
 *
 * @code
 * source:
 *   plugin: d7_node
 *   node_type: [page, test]
 * @endcode
 *
 * In this example nodes of type page and test are retrieved from the source
 * database.
 *
 * For additional configuration keys, refer to the parent classes.
 *
 * @see \Drupal\migrate\Plugin\migrate\source\SqlBase
 * @see \Drupal\migrate\Plugin\migrate\source\SourcePluginBase
 *
 * @MigrateSource(
 *   id = "mw_d7_node",
 *   source_module = "node"
 * )
 */
class MWD7Node extends Node {

  /**
   * The config of content type to import.
   *
   * @var string
   */
  protected $config;

  /**
   * The config of content type to import.
   *
   * @var string
   */
  protected $configFactory;

  /**
   * Array of bundles.
   *
   * @var array
   */
  public $bundle;

  /**
   * Array of nid or vid destiny type.
   *
   * @var string
   */
  protected $destinyType;

  /**
   * Array of machine name of origin to import.
   *
   * @var string
   */
  protected $originType;

  /**
   * The left join table.
   *
   * @var string
   */
  protected $leftJoinTable;

  /**
   * The current config of content to import.
   *
   * @var array
   */
  protected $currentConfig;


  /**
   * {@inheritdoc}
   */
  protected $fieldTypeMWManager;

  /**
   * {@inheritdoc}
   */
  protected $leftTable;

  /**
   * {@inheritdoc}
   */
  protected $leftTableId;

  /**
   * {@inheritdoc}
   */
  protected $entityType;

  /**
   * {@inheritdoc}
   */
  protected $translation;
  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration,
                              $plugin_id,
                              $plugin_definition,
                              MigrationInterface $migration,
                              StateInterface $state,
                              EntityTypeManagerInterface $entity_type_manager,
                              ModuleHandlerInterface $module_handler,
                              ConfigFactoryInterface $config_factory,
                              FieldTypeMWManagerInterface $field_type_mw_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration, $state, $entity_type_manager, $module_handler);

    $this->state = $state;
    $current_source_config = $migration->getSourceConfiguration();
    $this->config = $config_factory->getEditable('migrate_wizard.settings');
    $this->bundle = $current_source_config['bundle'];
    if(isset($current_source_config["constants"]["translations"]) && $current_source_config["constants"]["translations"]){
      $this->bundle = str_replace('_'.$current_source_config["constants"]["langcode"], '', $current_source_config['bundle']);
    }
    $this->currentConfig = $this->config->get($this->bundle);

    $this->translation = ['und', 'es'];
    if(isset($this->currentConfig[array_key_first($this->currentConfig)]["languages"])){
      $this->translation = ['und', $this->currentConfig[array_key_first($this->currentConfig)]["languages"]["default"]];
    }

    if(isset($configuration["constants"]["translations"]) && $configuration["constants"]["translations"]){
      $this->translation = [$configuration["constants"]["langcode"]];
    }
    $this->destinyType = $this->currentConfig["destiny_type"];
    $this->originType = $this->currentConfig["origin_type"];
    $this->configFactory = $config_factory;
    $this->fieldTypeMWManager = $field_type_mw_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration = NULL) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $migration,
      $container->get('state'),
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('config.factory'),
      $container->get('plugin.manager.field_type_mw')
    );
  }

  /**
   * The join options between the node and the node_revisions table.
   */
  const JOIN = '[n].[vid] = [nr].[vid]';

  /**
   * {@inheritdoc}
   */
  public function query() {

    $this->leftTable = 'n';
    $this->leftTableId = 'nid';
    $this->entityType = 'node';
    $this->leftJoinTable = $this->getLeftJoinTable();


    $query = parent::query();

    if($this->currentConfig["origin_type"] == 'node'){
      $query->condition('n.language', $this->translation, 'in');
    }

    if (isset($this->currentConfig["import_path_alias"]) && $this->currentConfig["import_path_alias"] == 1 && !array_key_exists('ual', $query->getTables())) {
      $this->queryEntityUrlAlias($query);
    }
    if (isset($this->currentConfig["source_type_date_field"]) && $this->currentConfig["source_type_date_field"] != NULL) {
      $this->addFromCreate($query, strtotime($this->currentConfig["source_type_date_field"]));
    }

    if (isset($this->currentConfig["import_only_ids"]) && $this->currentConfig["import_only_ids"] != NULL) {
      $this->importOnlyIds($query);
    }

    if (isset($this->currentConfig["limit_import"]) && $this->currentConfig["limit_import"] != NULL) {
      $this->limitImport($query);
    }

    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareRow(Row $row) {

    $prepare_row_result = parent::prepareRow($row);

    if($row->hasSourceProperty('translations') && $row->getSourceProperty('translations')){
       $row->setSourceProperty('default_langcode', 0);
    }
    // Get field API field values.
    foreach (array_keys($this->getFields($this->currentConfig["origin_type"], $this->bundle)) as $field) {
      if(isset(reset($this->currentConfig)[$field]) && (reset($this->currentConfig)[$field]['type'] == 'field_collection') || (reset($this->currentConfig)[$field]['type'] == 'paragraphs')){
        $new_field_source = [];
        $field_source = $row->getSourceProperty($field);
        foreach($field_source as $field_source_key => $field_source_value){
          $new_field_source[$field_source_key] = ['item_id' => $field_source_value['value'], 'revision_id' => $field_source_value['revision_id']];
        }
        $row->setSourceProperty($field, $new_field_source);
      }
    }

    foreach ($this->currentConfig[array_key_first($this->currentConfig)] as $field_name => $data_field) {
      if(isset($data_field['type']) && $data_field["destiny"] != 'none'){
        switch ($data_field['type']) {
          case 'image':
            $new_value = [];
            foreach($row->getSourceProperty($field_name) as $key => $value){
              $query_uri = $this->select('file_managed', 'fm');
              $query_uri->fields('fm', ['uri']);
              $query_uri->leftJoin('field_data_field_file_image_alt_text', "ffat", "fm.fid = ffat.entity_id");
              $query_uri->fields('ffat', ['field_file_image_alt_text_value']);
              $query_uri->leftJoin('field_data_field_file_image_title_text', "fftt", "fm.fid = fftt.entity_id");
              $query_uri->fields('fftt', ['field_file_image_title_text_value']);
              $query_uri->condition('fm.fid', $value['fid']);
              $results = $query_uri->execute()->fetch();
              $new_value[$key] = $value;
              if ($results["field_file_image_alt_text_value"]) {
                $new_value[0]["alt"] = $results["field_file_image_alt_text_value"];
              }
              if ($results["field_file_image_title_text_value"]) {
                $new_value[0]["title"] = $results["field_file_image_title_text_value"];
              }
              $new_value[$key]['uri'] = $results['uri'];
              $new_value[$key]['type'] = $data_field['type'];
            }
            $row->setSourceProperty($field_name, $new_value);
            break;
          case 'file':
            $new_value = [];
            foreach($row->getSourceProperty($field_name) as $key => $value){
              $query_uri = $this->select('file_managed', 'fm');
              $query_uri->fields('fm', ['uri']);
              $query_uri->condition('fm.fid', $value['fid']);
              $uri = $query_uri->execute()->fetchField();
              $new_value[$key] = $value;
              $new_value[$key]['uri'] = $uri;
              $new_value[$key]['type'] = $data_field['type'];
            }
            $row->setSourceProperty($field_name, $new_value);
            break;
          case 'datetime':
            $date = $row->getSourceProperty($field_name . '_value');
            if (isset($data_field['format']) && $data_field['format'] != '' && $date != '') {
              $date = DrupalDateTime::createFromFormat('Y-m-d H:i:s', $date);
              $date->format($data_field['format']);
              $row->setSourceProperty($field_name . '_value', $date->format($data_field['format']));

            }
            break;
          case 'list_text':
            if ($data_field['destiny_type'] == 'entity_reference') {
              $this->listTextToTaxonomy($row, $field_name, $data_field);
            }
            break;
          case 'entityreference':
            if ($data_field['destiny_type'] == 'entity_reference_revisions') {
              $config_dependency = $this->config->get(reset($data_field["dependencies"]));
              if($config_dependency["origin_type"] == 'node' && $config_dependency["destiny_type"] == 'paragraph'){
                $new_values = [];
                $values = $row->getSourceProperty($field_name);
                foreach($values as $key => $value){
                  $new_values[$key] = ['nid' => reset($value)];
                }
                $row->setSourceProperty($field_name, $new_values);
              }
            }
            break;
        }
      }
    }

    return $prepare_row_result;
  }

  /**
   * {@inheritdoc}
   */
  protected function queryEntityUrlAlias(&$query): void
  {

    $translations = implode("','", $this->translation);
    $id__left = $this->leftJoinTable;
    if($this->leftJoinTable == 'tv.vid'){
      $id__left = 'ttd.tid';
    }

    $source = $this->entityType;
    if($this->entityType == 'taxonomy_term'){
      $source = 'taxonomy/term';
    }

    $query->leftJoin('url_alias', 'ual', "CONCAT('{$source}/', $id__left) = ual.source AND ual.pid = (SELECT MAX(ual2.pid) FROM url_alias as ual2 WHERE ual2.source=CONCAT('{$source}/', $id__left)) AND ual.language in('$translations')");
    $query->fields('ual', ['alias']);
    $query->groupBy('ual.alias');
  }

  /**
   * {@inheritdoc}
   */
    protected function addFromCreate(&$query, $timestamp): void
  {
    $query->condition('created', $timestamp, '>=');
  }

  /**
   * {@inheritdoc}
   */
    protected function importOnlyIds(&$query): void
  {
    switch($this->currentConfig["origin_type"]){
      case 'node':
        $query->condition('n.nid', explode(',', $this->currentConfig["import_only_ids"]), 'IN');
        break;
      case 'taxonomy_term':
        $query->condition('ttd.tid', explode(',', $this->currentConfig["import_only_ids"]), 'IN');
        break;
      case 'user':
        $query->condition('u.uid', explode(',', $this->currentConfig["import_only_ids"]), 'IN');
        break;
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function limitImport(&$query): void
  {
    $query->range(0, $this->currentConfig["limit_import"]);

    $query->orderBy($this->leftJoinTable, 'ASC');
  }

  /**
   * {@inheritdoc}
   */
  protected function getLeftJoinTable(): string
  {
    return "$this->leftTable.$this->leftTableId";
  }

  /**
   * {@inheritdoc}
   */
  private function listTextToTaxonomy(&$row, $field_name, $data_field): void {
    // Load taxonomy term.
    $term = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadByProperties(['name' => $row->getSourceProperty($field_name . '_value')]);
    $term = reset($term);
    if ($term == FALSE) {
      $term = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->create([
        'name' => $row->getSourceProperty($field_name . '_value'),
        'vid' => $data_field['destiny'],
      ]);
      $term->save();
    }
    $row->setSourceProperty($field_name . '_value', $term->id());
  }

  public function getIds() {
    $ids = [];
//    switch($this->destinyType){
//      case 'node':
        $ids['nid']['type'] = 'integer';
        $ids['nid']['alias'] = 'n';
//        break;
//      case 'paragraph':
//        $ids['target_id']['type'] = 'integer';
//        $ids['target_id']['alias'] = 'tid';
//        break;
//    }

    return $ids;
  }
}
